<?php
namespace Teknisa\EST\Modules\EstoqueCego\Controllers;

use Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use \Teknisa\Libs\Util\DateUtil;
use \Geral\Entities\Paramfilial;

use Zeedhi\Framework\Report\ReportService;

class Est95100AnaliseInventarioCegoController extends TeknisaCrud {

    protected $dataSourceName = "";
    protected $dataSourceManager;
    protected $environment;
    protected $crudService;
    protected $reportService;
    
    protected $nrorg;
    protected $currentTime;
    protected $operador;
    protected $filialLogada;
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, ReportService $reportService, $validacoesEstoqueService) {
        $this->crudService = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->reportService       = $reportService;
        
        $this->operador = $environment->getCdOperador();
        $this->nrorg = $environment->getNrOrg();
        $this->filialLogada = $environment->getCdFilial();
        $this->currentTime = $environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->environment = $environment;
    }
    
//termninar definição de parametros / verificação de relartórios
    public function generateReport(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        $reportName = $row['NMRELATORIO'];
        $reportStrategy = $this->getReportStrategyReport($reportName);
        var_dump($row);die;
        $parametersRow = array();
        
        foreach ($reportStrategy['parameterMapping'] as $fieldName => $paramName) {
            if ($row->has($fieldName)) {
                if($reportName === 'EST31119' || $reportName === 'EST32800' || $reportName === 'EST32804' || $reportName === 'EST32808' )  {
                    if($fieldName === 'NVTOT') {
                        $parametersRow[$paramName] = $this->getNvTot($row->get($fieldName));
                    }
                    else if($fieldName === 'ORDENANOME')  $parametersRow[$paramName]  = ($row->get($fieldName) == 'S') ? '0' : '10';
                    else if($fieldName === 'CDALMOXARIFE') $parametersRow[$paramName] = ($row->get($fieldName) == ' ') ? '' : $row->get($fieldName);
                    else if($fieldName === 'CDLOCALESTOQ') $parametersRow[$paramName] = ($row->get($fieldName) == ' ') ? '' : $row->get($fieldName);
                    else if($fieldName === 'DATAPOS')     {
                        $data = str_replace('/', '-', $row->get($fieldName));
                        if($reportName == 'EST31119') {
                            $data = date('m/d/Y', strtotime($data));
                        }
                        else {
                            $data = date('d/m/Y', strtotime($data));
                        }
                        $parametersRow[$paramName] = $data;
                        if($reportName === 'EST32804'  || $reportName === 'EST32808' ) {
                            $parametersRow['P_DTCUSTPROD'] = $data; 
                            $parametersRow['P_CDGRUPFILI'] = $row->get('CDGRUPFILI');
                        }
                    }
                    else $parametersRow[$paramName] = $row->get($fieldName);
                }
                else $parametersRow[$paramName] = $row->get($fieldName);
            }
        }
        $parametersRow['P_NRORG']  = $this->nrorg;
        
        $parametersRow['__format'] = $row["__FORMAT"];
        
        $birtURL = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow);
        
        $response->addMessage(new DTO\Response\Message($birtURL));
    }
    
    private function getReportStrategyReport($reportName) {
        $reportStrategy = array (
            'strategy' => 'BIRT'
        );
        switch($reportName) {
            case 'EST951001':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "CDFILIAL",
                    "CDALMOXARIFE" => "CDALMOXARIFE",
                    "CDLOCALESTOQ" => "CDLOCALESTOQ",
                    "NVTOT" => "NVTOT",
                    "ORDENACAO" => "ORDENACAO",
                    "CDARVPRODINI" => "CDARVPRODINI",
                    "CDARVPRODFIN" => "CDARVPRODFIN",
                    "DTINICIAL" => "DTINICIAL",
                    "DTFINAL" => "DTFINAL",
                    "IDSITUACEITEINV" => "IDSITUACEITEINV",
                    "UTILALMOX" => "UTILALMOX",
                    "CDOPERADOR" => "CDOPERADOR"
                );
                break;
            case 'EST951002':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "CDFILIAL",
                    "CDALMOXARIFE" => "CDALMOXARIFE",
                    "CDLOCALESTOQ" => "CDLOCALESTOQ",
                    "CDGRUPPROD" => "CDGRUPPROD",
                    "CDARVPRODINI" => "PRODINI",
                    "CDARVPRODFIN" => "PRODFIN",
                    "DTINICIAL" => "DTINICIAL",
                    "DTFINAL" => "DTFINAL",
                    "IDSITUACEITEINV" => "IDSITUACEITEINV",
                    "UTILALMOX" => "UTILALMOX",
                    "CDOPERADOR" => "CDOPERADOR"
                );
                break;
        }
        return $reportStrategy;
    }

    private function getNvTot($nivelEscolhido) {
        $nivelEscolhido  = $nivelEscolhido - 1;
        $filterCriteria  = new FilterCriteria('paramgeral',array());
        $dataSet         = $this->dataSourceManager->findBy($filterCriteria);
        $mascaraProdutos = $dataSet->getRows()[0]['CDPICTPROD'];
        $arrayMascara    = explode(".", $mascaraProdutos);
        for($count = 0, $nvTot = 0; $count <= $nivelEscolhido; $count++) {
            $nvTot += strlen($arrayMascara[$count]);
        }
        return $nvTot;
    }
}
